/*
 *   GtkAudioScale
 *    
 *   Copyright (C) Robert Ham 2002 (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>
#include <strings.h>


#include "gtkaudioscale/audioscale.h"

#define _(x) x

#define AUDIO_SCALE_DEFAULT_WIDTH 300
#define AUDIO_SCALE_DEFAULT_HEIGHT 50

static void gtk_audio_scale_class_init (GtkAudioScaleClass * class);
static void gtk_audio_scale_init (GtkAudioScale * scale);
static void gtk_audio_scale_set_property   (GObject          *object,
                                            guint             prop_id,
                                            const GValue     *value,
                                            GParamSpec       *pspec);
static void gtk_audio_scale_get_property   (GObject          *object,
                                            guint             prop_id,
                                            GValue           *value,
                                            GParamSpec       *pspec);
static void gtk_audio_scale_destroy        (GtkObject        *object);
static void gtk_audio_scale_adjustment_changed (GtkAdjustment *, gpointer);
static void gtk_audio_scale_adjustment_value_changed (GtkAdjustment *, gpointer);
static void gtk_audio_scale_realize (GtkWidget *widget);
static void gtk_audio_scale_size_request (GtkWidget *widget, GtkRequisition *requisition);
static void gtk_audio_scale_finalise (GObject * object);
static void gtk_audio_scale_size_allocate (GtkWidget     *widget, GtkAllocation *allocation);
static gint gtk_audio_scale_expose (GtkWidget      *widget, GdkEventExpose *event);


/* signals */
enum {
  ZOOM_UPPER_CHANGED,
  ZOOM_LOWER_CHANGED,
  VALUE_CHANGED,
  LAST_SIGNAL
};

/* properties */
enum {
  PROP_0,
  PROP_LOGARITHMIC,
  PROP_SCALE_VISIBLE,
/*  PROP_SCALE_POLICY,
  PROP_SCALE_TICKS, */
  PROP_ZOOM_UPPER,
  PROP_ZOOM_LOWER,
  PROP_ADJUSTMENT,
  PROP_ZOOM_UPPER_STYLE,
  PROP_ZOOM_LOWER_STYLE,
  PROP_ZOOM_INDICATOR_SIZE,
  PROP_ORIENTATION
};

static guint audio_scale_signals[LAST_SIGNAL] = { 0 };
static GtkWidgetClass *parent_class = NULL;

GType
gtk_audio_scale_get_type ()
{
  static GType audio_scale_type = 0;
  
  if (!audio_scale_type)
    {
      static const GTypeInfo audio_scale_info =
      {
        sizeof (GtkAudioScaleClass),
        NULL,
        NULL,
        (GClassInitFunc) gtk_audio_scale_class_init,
        NULL,
        NULL,
        sizeof (GtkAudioScale),
        0,
        (GInstanceInitFunc) gtk_audio_scale_init
      };

      audio_scale_type = g_type_register_static (GTK_TYPE_WIDGET,
                                                 "GtkAudioScale",
                                                 &audio_scale_info,
                                                 G_TYPE_FLAG_ABSTRACT);
    }
  
  return audio_scale_type;
}

static void
gtk_audio_scale_class_init (GtkAudioScaleClass * class)
{
  GObjectClass * gobject_class;
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;
      
  parent_class = g_type_class_peek_parent (class);
  gobject_class = G_OBJECT_CLASS (class);
  object_class = (GtkObjectClass*) class;
  widget_class = (GtkWidgetClass*) class;
  
  gobject_class->set_property = gtk_audio_scale_set_property;
  gobject_class->get_property = gtk_audio_scale_get_property;
  gobject_class->finalize = gtk_audio_scale_finalise;
  
  object_class->destroy = gtk_audio_scale_destroy;
  
  widget_class->size_request = gtk_audio_scale_size_request;
  widget_class->realize = gtk_audio_scale_realize;
  widget_class->size_allocate = gtk_audio_scale_size_allocate;
  widget_class->expose_event = gtk_audio_scale_expose;

  /* no default handlers */
  class->zoom_upper_changed = NULL;
  class->zoom_lower_changed = NULL;
  class->value_changed = NULL;
      
  /*
   * properties
   */
  g_object_class_install_property (gobject_class,
                                   PROP_LOGARITHMIC,
                                   g_param_spec_boolean ("logarithmic",
                                                         _("Logarithmic"),
                                                         _("Whether or not the scale is logarithmic"),
                                                         FALSE,
                                                         G_PARAM_READWRITE));
                                                         
  g_object_class_install_property (gobject_class,
                                   PROP_SCALE_VISIBLE,
                                   g_param_spec_boolean ("scale_visible",
                                                         _("Scale visible"),
                                                         _("Whether or not to draw the scale"),
                                                         TRUE,
                                                         G_PARAM_READWRITE));

  g_object_class_install_property (gobject_class,
                                   PROP_ZOOM_UPPER,
                                   g_param_spec_double ("zoom_upper",
                                                        _("Zoom Upper"),
                                                        _("The upper zoom limit"),
                                                        G_MINDOUBLE,
                                                        G_MAXDOUBLE,
                                                        1.0,
                                                        G_PARAM_READWRITE));

  g_object_class_install_property (gobject_class,
                                   PROP_ZOOM_UPPER,
                                   g_param_spec_double ("zoom_lower",
                                                        _("Zoom Lower"),
                                                        _("The lower zoom limit"),
                                                        G_MINDOUBLE,
                                                        G_MAXDOUBLE,
                                                        1.0,
                                                        G_PARAM_READWRITE));

  g_object_class_install_property (gobject_class,
                                   PROP_ADJUSTMENT,
                                   g_param_spec_object ("adjustment",
                                                        _("Adjustment"),
                                                        _("A GtkAdjustment containing the value and step sizes, etc, for this widget"),
                                                        GTK_TYPE_ADJUSTMENT,
                                                        G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));

  g_object_class_install_property (gobject_class,
                                   PROP_ZOOM_UPPER_STYLE,
                                   g_param_spec_object ("zoom_upper_style",
                                                        _("Zoom Upper style"),
                                                        _("A GtkStyle that defines the colour to be used drawing the upper zoom indicator"),
                                                        GTK_TYPE_STYLE,
                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT));

  g_object_class_install_property (gobject_class,
                                   PROP_ZOOM_LOWER_STYLE,
                                   g_param_spec_object ("zoom_lower_style",
                                                        _("Zoom Lower style"),
                                                        _("A GtkStyle that defines the colour to be used drawing the lower zoom indicator"),
                                                        GTK_TYPE_STYLE,
                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT));

  g_object_class_install_property (gobject_class,
                                   PROP_ZOOM_INDICATOR_SIZE,
                                   g_param_spec_uint ("zoom_indicator_size",
                                                      _("Zoom indicator size"),
                                                      _("The size of the zoom indicators"),
                                                      0,
                                                      G_MAXUINT,
                                                      4,
                                                      G_PARAM_READWRITE));
                                                         
  g_object_class_install_property (gobject_class,
                                   PROP_ORIENTATION,
                                   g_param_spec_enum ("orientation",
                                                      _("Orientation"),
                                                      _("How to draw the widget; vertical or horizontal"),
                                                      GTK_TYPE_ORIENTATION,
                                                      GTK_ORIENTATION_HORIZONTAL,
                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
                                                         

  /*
   * signals
   */
  audio_scale_signals[ZOOM_UPPER_CHANGED] =
    g_signal_new ("zoom-upper-changed",
                  gtk_audio_scale_get_type (),
                  G_SIGNAL_RUN_FIRST,
                  G_STRUCT_OFFSET (GtkAudioScaleClass, zoom_upper_changed),
                  NULL, /* signal accumulator callback */
                  NULL, /* data for accumulator */
                  g_cclosure_marshal_VOID__VOID,
                  G_TYPE_NONE, /* return type */
                  0);
                  
  audio_scale_signals[ZOOM_LOWER_CHANGED] =
    g_signal_new ("zoom-lower-changed",
                  gtk_audio_scale_get_type (),
                  G_SIGNAL_RUN_FIRST,
                  G_STRUCT_OFFSET (GtkAudioScaleClass, zoom_lower_changed),
                  NULL, /* signal accumulator callback */
                  NULL, /* data for accumulator */
                  g_cclosure_marshal_VOID__VOID,
                  G_TYPE_NONE, /* return type */
                  0);
                  
  audio_scale_signals[VALUE_CHANGED] =
    g_signal_new ("value-changed",
                  gtk_audio_scale_get_type (),
                  G_SIGNAL_RUN_FIRST,
                  G_STRUCT_OFFSET (GtkAudioScaleClass, value_changed),
                  NULL, /* signal accumulator callback */
                  NULL, /* data for accumulator */
                  g_cclosure_marshal_VOID__VOID,
                  G_TYPE_NONE, /* return type */
                  0);
}

static void
gtk_audio_scale_init (GtkAudioScale * scale)
{
  scale->adjustment = NULL;
  scale->logarithmic = FALSE;
  scale->orientation = GTK_ORIENTATION_HORIZONTAL;
  scale->zoom_upper = 0.0;
  scale->zoom_lower = 0.0;
  scale->adjustment_mode = GTK_AUDIO_SCALE_NORMAL;
  scale->zoom_upper_style = NULL;
  scale->zoom_lower_style = NULL;
  scale->zoom_indicator_size = 4;
  scale->scale_visible = TRUE;
  scale->adjustment_handlers[0] = 0;
  scale->adjustment_handlers[1] = 0;
  scale->mouse_position = GTK_AUDIO_SCALE_OUTSIDE;
  scale->major_tick = 4;
  scale->minor_tick = 2;
}

static void
gtk_audio_scale_finalise (GObject * object)
{
  GtkAudioScale * scale;
  
  g_return_if_fail (GTK_IS_AUDIO_SCALE (object));
  
  scale = GTK_AUDIO_SCALE (object);
  
  if (scale->zoom_upper_style)
    {
      gtk_style_detach (scale->zoom_upper_style);
    }

  if (scale->zoom_lower_style)
    {
      gtk_style_detach (scale->zoom_lower_style);
    }
}

static void
gtk_audio_scale_destroy (GtkObject *object)
{
  GtkAudioScale * scale;

  g_return_if_fail (GTK_IS_AUDIO_SCALE (object));
  
  scale = GTK_AUDIO_SCALE (object);

  if (scale->adjustment)
    g_object_unref (scale->adjustment); 
  
  if (scale->zoom_upper_style)
    {
      g_object_unref (scale->zoom_upper_style);
    }

  if (scale->zoom_lower_style)
    {
      g_object_unref (scale->zoom_lower_style);
    }

 if (GTK_OBJECT_CLASS (parent_class)->destroy)
   (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

static void
gtk_audio_scale_set_property   (GObject          *object, 
                                guint             prop_id,
                                const GValue     *value, 
                                GParamSpec       *pspec)
{
  GtkAudioScale * scale;

  scale = (GtkAudioScale *) object;

  switch (prop_id)
    {
    case PROP_LOGARITHMIC:
      gtk_audio_scale_set_logarithmic (scale, g_value_get_boolean (value));
      break;
    case PROP_SCALE_VISIBLE:
      gtk_audio_scale_set_scale_visible (scale, g_value_get_boolean (value));
      break;
    case PROP_ZOOM_UPPER:
      gtk_audio_scale_set_zoom_upper (scale, g_value_get_double (value));
      break;
    case PROP_ZOOM_LOWER:
      gtk_audio_scale_set_zoom_lower (scale, g_value_get_double (value));
      break;
    case PROP_ADJUSTMENT:
      gtk_audio_scale_set_adjustment (scale, g_value_get_object (value));
      break;
    case PROP_ZOOM_UPPER_STYLE:
      gtk_audio_scale_set_zoom_upper_style (scale, g_value_get_object (value));
      break;
    case PROP_ZOOM_LOWER_STYLE:
      gtk_audio_scale_set_zoom_lower_style (scale, g_value_get_object (value));
      break;
    case PROP_ZOOM_INDICATOR_SIZE:
      gtk_audio_scale_set_zoom_indicator_size (scale, g_value_get_uint (value));
      break;
    case PROP_ORIENTATION:
      gtk_audio_scale_set_orientation (scale, g_value_get_enum (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
gtk_audio_scale_get_property   (GObject          *object, 
                                guint             prop_id,
                                GValue           *value, 
                                GParamSpec       *pspec)
{
  GtkAudioScale * scale;
  
  scale = (GtkAudioScale *) object;
 
  switch (prop_id)
    {
    case PROP_LOGARITHMIC:
      g_value_set_boolean (value, scale->logarithmic);
      break;
    case PROP_SCALE_VISIBLE:
      g_value_set_boolean (value, scale->scale_visible);
      break;
    case PROP_ZOOM_UPPER:
      g_value_set_double (value, scale->zoom_upper);
      break;
    case PROP_ZOOM_LOWER:
      g_value_set_double (value, scale->zoom_lower);
      break;
    case PROP_ZOOM_UPPER_STYLE:
      g_value_set_object (value, scale->zoom_upper_style);
      break;
    case PROP_ZOOM_LOWER_STYLE:
      g_value_set_object (value, scale->zoom_lower_style);
      break;
    case PROP_ZOOM_INDICATOR_SIZE:
      g_value_set_uint (value, scale->zoom_indicator_size);
      break;
    case PROP_ORIENTATION:
      g_value_set_enum (value, scale->orientation);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/* attempt to find a decent font to draw the scale text with, and
 * set the widget's font property accordingly */
static void
gtk_audio_scale_find_decent_font (GtkAudioScale * scale)
{
  PangoFontDescription * desc, * font_desc;
  PangoContext * context;
  PangoFont * font;
  guint i;
  struct {
    const gchar * family;
    gint size;
  } ideal_fonts[] = { { "Clean", 6 },
                      { "New Century Schoolbook", 8 },
                      { "Symbol", 8 },
                      { "Lucida Typewriter", 8 },
                      { "Luxi Sans", 8 },
                      { "Lucida", 8 },
                      { "Luxi Mono", 8 },
                      { "Helvetica", 8 },
                      { NULL, 0 } };
  
  context = gtk_widget_get_pango_context (GTK_WIDGET (scale));
  
  /* see which one we can load, if any */
  for (i = 0; ideal_fonts[i].family; ++i)
    {
    
      desc = pango_font_description_new ();
      pango_font_description_set_family (desc, ideal_fonts[i].family);
      pango_font_description_set_size (desc, ideal_fonts[i].size * PANGO_SCALE);
    
      font = pango_context_load_font (context, desc);
      
      if (font)
        {
          font_desc = pango_font_describe (font);
          
          g_object_unref (font);
          
          if (strcasecmp (pango_font_description_get_family (desc),
                          pango_font_description_get_family (font_desc)) == 0 &&
              pango_font_description_get_size (desc) ==
              pango_font_description_get_size (font_desc))
            {
              pango_font_description_free (font_desc);
              gtk_widget_modify_font (GTK_WIDGET (scale), desc);
              return;
            }
          
          pango_font_description_free (font_desc);
        }
      pango_font_description_free (desc);
    }
  
}


static void
gtk_audio_scale_adjustment_value_changed (GtkAdjustment * adj, gpointer data)
{
  GtkAudioScale * scale;
  
  g_return_if_fail (GTK_IS_AUDIO_SCALE (data));
  
  scale = data;
  
  /* FIXME: fill in */
}

static void
gtk_audio_scale_adjustment_changed (GtkAdjustment * adj, gpointer data)
{
  GtkAudioScale * scale;
  
  g_return_if_fail (GTK_IS_AUDIO_SCALE (data));
  
  scale = data;
  
  /* FIXME: fill in */
}

static void
gtk_audio_scale_realize (GtkWidget *widget)
{
  GtkAudioScale * scale;
  GdkWindowAttr attributes;
  gint attributes_mask;

  g_return_if_fail (GTK_IS_AUDIO_SCALE (widget));
  
  scale = GTK_AUDIO_SCALE (widget);

  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);

  attributes.x = widget->allocation.x;
  attributes.y = widget->allocation.y;
  attributes.width = widget->allocation.width;
  attributes.height = widget->allocation.height;
  attributes.wclass = GDK_INPUT_OUTPUT;
  attributes.window_type = GDK_WINDOW_CHILD;
  attributes.event_mask = gtk_widget_get_events (widget) | 
    GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | 
    GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK |
    GDK_POINTER_MOTION_HINT_MASK;
  attributes.visual = gtk_widget_get_visual (widget);
  attributes.colormap = gtk_widget_get_colormap (widget);

  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
  widget->window = gdk_window_new (widget->parent->window, &attributes, attributes_mask);
/*  widget->window = gtk_widget_get_parent_window (widget);*/
  g_object_ref (widget->window);

  widget->style = gtk_style_attach (widget->style, widget->window);

  scale->zoom_upper_style = gtk_style_attach (scale->zoom_upper_style, widget->window);
  scale->zoom_lower_style = gtk_style_attach (scale->zoom_lower_style, widget->window);
  
  gtk_style_set_background (widget->style, widget->window, GTK_STATE_ACTIVE);
  
  gdk_window_set_user_data (widget->window, widget);
  
  gtk_audio_scale_find_decent_font (scale);
}

static void 
gtk_audio_scale_size_request (GtkWidget      *widget,
	  	              GtkRequisition *requisition)
{
  GtkAudioScale * scale;
  
  g_return_if_fail (GTK_IS_AUDIO_SCALE (widget));
  
  scale = GTK_AUDIO_SCALE (widget);
  
  if (scale->orientation == GTK_ORIENTATION_HORIZONTAL)
    {
      requisition->width = AUDIO_SCALE_DEFAULT_WIDTH;
      requisition->height = AUDIO_SCALE_DEFAULT_HEIGHT;
    }
  else
    {
      requisition->width = AUDIO_SCALE_DEFAULT_HEIGHT;
      requisition->height = AUDIO_SCALE_DEFAULT_WIDTH;
    }
}

static void
gtk_audio_scale_size_allocate (GtkWidget     *widget,
                               GtkAllocation *allocation)
{
  g_return_if_fail (GTK_IS_AUDIO_SCALE (widget));
  
  widget->allocation = *allocation;
  if (GTK_WIDGET_REALIZED(widget))
    {
      gdk_window_move_resize (widget->window,
      			      allocation->x, allocation->y,
      			      allocation->width, allocation->height);
    }
}

/*static void
gtk_audio_scale_map (GtkWidget *widget)
{
  GtkAudioScale *range = GTK_AUDIO_SCALE (widget);
  
  g_return_if_fail (GTK_IS_AUDIO_SCALE (widget));

  gdk_window_show (range->event_window);

  GTK_WIDGET_CLASS (parent_class)->map (widget);
}*/

static void
gtk_audio_scale_calculate_layout (GtkAudioScale * scale)
{
  
}

static gint
gtk_audio_scale_expose (GtkWidget      *widget,
                        GdkEventExpose *event)
{
  PangoLayout * layout;
  GtkAudioScale * scale;
  gchar * str;
  gfloat flt;
  gint trough_thickness, major_tick, minor_tick, scale_height;
  gint layout_w, layout_h;
  gboolean sensitive;
  
  g_return_val_if_fail (GTK_IS_AUDIO_SCALE (widget), FALSE);
  
  scale = GTK_AUDIO_SCALE (widget);
  
  sensitive = GTK_WIDGET_IS_SENSITIVE (widget);
  

  /*
   * find the sizes of everything
   */
   
  /* make a layout of the upper limit's scale text in order to get a scale height */
  str = g_strdup_printf ("%.3g", scale->zoom_upper);
  layout = gtk_widget_create_pango_layout (widget, str);
  pango_layout_get_pixel_size (layout, &layout_w, &layout_h);
  g_object_unref (G_OBJECT(layout));
  scale_height = layout_h;
  
  major_tick = 4;
  minor_tick = 2;
  trough_thickness = (scale->orientation == GTK_ORIENTATION_HORIZONTAL ?
                       widget->allocation.height :
                       widget->allocation.width)
                     - scale->zoom_indicator_size
                     - major_tick
                     - scale_height - 2;
  
  gtk_paint_box (widget->style,
                 widget->window,
                 GTK_STATE_ACTIVE,
                 GTK_SHADOW_IN,
                 NULL,
                 widget,
                 "audioscale",
                 scale->orientation == GTK_ORIENTATION_HORIZONTAL ?
                   1 :
                   scale->zoom_indicator_size,
                 scale->orientation == GTK_ORIENTATION_HORIZONTAL ?
                   scale->zoom_indicator_size :
                   1,
                 scale->orientation == GTK_ORIENTATION_HORIZONTAL ?
                   widget->allocation.width - 2 :
                   trough_thickness,
                 scale->orientation == GTK_ORIENTATION_HORIZONTAL ?
                   trough_thickness :
                   widget->allocation.width - 2);
                 
  
  gtk_paint_layout (widget->style,
                          widget->window,
                          GTK_STATE_NORMAL,
                          FALSE,
                          NULL,
                          widget,
                          "audioscale",
                          10,
                          10,
                          layout);
  

  
  if (GTK_WIDGET_CLASS (parent_class)->expose_event)
      GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
  
  return FALSE;
}

/*
 *
 * User visible functions
 *
 */
gboolean
gtk_audio_scale_get_logarithmic (GtkAudioScale * scale)
{
  g_return_val_if_fail (GTK_IS_AUDIO_SCALE (scale), FALSE);
  
  return scale->logarithmic;
}

void
gtk_audio_scale_set_logarithmic (GtkAudioScale * scale, gboolean logarithmic)
{
  g_return_if_fail (GTK_IS_AUDIO_SCALE (scale));
  
  if (logarithmic == scale->logarithmic)
    return;

  if (scale->logarithmic)
    {
      scale->zoom_upper = exp (scale->zoom_upper);
      scale->zoom_lower = exp (scale->zoom_lower);
      scale->adjustment->upper = exp (scale->adjustment->upper);
      scale->adjustment->lower = exp (scale->adjustment->lower);
      scale->adjustment->value = exp (scale->adjustment->value);
      scale->adjustment->step_increment = exp (scale->adjustment->step_increment);
      scale->adjustment->page_increment = exp (scale->adjustment->page_increment);
      scale->adjustment->page_size = exp (scale->adjustment->page_size);
    }
  else
    {
      scale->zoom_upper = log (scale->zoom_upper);
      scale->zoom_lower = log (scale->zoom_lower);
      scale->adjustment->upper = log (scale->adjustment->upper);
      scale->adjustment->lower = log (scale->adjustment->lower);
      scale->adjustment->value = log (scale->adjustment->value);
      scale->adjustment->step_increment = log (scale->adjustment->step_increment);
      scale->adjustment->page_increment = log (scale->adjustment->page_increment);
      scale->adjustment->page_size = log (scale->adjustment->page_size);
    }
  scale->logarithmic = logarithmic;
  g_object_notify (G_OBJECT (scale), "logarithmic");
  
  gtk_adjustment_changed (scale->adjustment);
  gtk_adjustment_value_changed (scale->adjustment);
}

gboolean
gtk_audio_scale_get_scale_visible (GtkAudioScale * scale)
{
  g_return_val_if_fail (GTK_IS_AUDIO_SCALE (scale), FALSE);
  
  return scale->scale_visible;
}

void
gtk_audio_scale_set_scale_visible (GtkAudioScale * scale, gboolean scale_visible)
{
  g_return_if_fail (GTK_IS_AUDIO_SCALE (scale));

  scale->scale_visible = scale_visible;
  g_object_notify (G_OBJECT (scale), "scale_visible");
}

gdouble
gtk_audio_scale_get_zoom_upper (GtkAudioScale * scale)
{
  g_return_val_if_fail (GTK_IS_AUDIO_SCALE (scale), 0.0);
  
  if (scale->logarithmic)
    return exp (scale->zoom_upper);
  else  
    return scale->zoom_upper;
}

void
gtk_audio_scale_set_zoom_upper (GtkAudioScale * scale, gdouble zoom_upper)
{
  g_return_if_fail (GTK_IS_AUDIO_SCALE (scale));

  if (scale->logarithmic)
    scale->zoom_upper = log (zoom_upper);
  else  
    scale->zoom_upper = zoom_upper;
    
  g_object_notify (G_OBJECT (scale), "zoom_upper");
}

gdouble
gtk_audio_scale_get_zoom_lower (GtkAudioScale * scale)
{
  g_return_val_if_fail (GTK_IS_AUDIO_SCALE (scale), 0.0);
  
  if (scale->logarithmic)
    return exp (scale->zoom_lower);
  else  
    return scale->zoom_lower;
}

void
gtk_audio_scale_set_zoom_lower (GtkAudioScale * scale, gdouble zoom_lower)
{
  g_return_if_fail (GTK_IS_AUDIO_SCALE (scale));
  
  if (scale->logarithmic)
    scale->zoom_lower = log (zoom_lower);
  else 
    scale->zoom_lower = zoom_lower;
  
  g_object_notify (G_OBJECT (scale), "zoom_lower");
}

/*GtkAdjustment *
gtk_audio_scale_get_adjustment		(GtkAudioScale * scale)
{
  g_return_val_if_fail (GTK_IS_AUDIO_SCALE (scale), NULL);
  
  return scale->adjustment;
}*/

void
gtk_audio_scale_set_adjustment (GtkAudioScale * scale, GtkAdjustment * adjustment)
{
  g_return_if_fail (GTK_IS_AUDIO_SCALE (scale));
  g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
  
  if (scale->adjustment != adjustment)
    {
      if (scale->adjustment)
        {
          g_signal_handler_disconnect (scale->adjustment, scale->adjustment_handlers[0]);
          g_signal_handler_disconnect (scale->adjustment, scale->adjustment_handlers[1]);
      
          g_object_unref (G_OBJECT(scale->adjustment));
        }
  
      scale->adjustment = adjustment;
      g_object_ref (G_OBJECT(adjustment));
      gtk_object_sink (GTK_OBJECT(adjustment));
      
      scale->adjustment_handlers[0] = g_signal_connect (G_OBJECT (adjustment), "value-changed",
                                                        G_CALLBACK (gtk_audio_scale_adjustment_value_changed),
                                                        scale);
      scale->adjustment_handlers[1] = g_signal_connect (G_OBJECT (adjustment), "changed",
                                                        G_CALLBACK (gtk_audio_scale_adjustment_changed),
                                                        scale);
      
      if (scale->logarithmic)
        {
          scale->adjustment->upper = log (scale->adjustment->upper);
          scale->adjustment->lower = log (scale->adjustment->lower);
          scale->adjustment->value = log (scale->adjustment->value);
          scale->adjustment->step_increment = log (scale->adjustment->step_increment);
          scale->adjustment->page_increment = log (scale->adjustment->page_increment);
          scale->adjustment->page_size = log (scale->adjustment->page_size);
        }
      
      gtk_audio_scale_adjustment_changed (adjustment, scale);
      g_object_notify (G_OBJECT (scale), "adjustment");
    }
}

GtkStyle *
gtk_audio_scale_get_zoom_upper_style        (GtkAudioScale * scale)
{
  g_return_val_if_fail (GTK_IS_AUDIO_SCALE (scale), NULL);
  
  return scale->zoom_upper_style;
}

void
gtk_audio_scale_internal_set_zoom_style (GtkAudioScale * scale,
                                         GtkStyle * style,
                                         GtkStyle ** styleptr,
                                         const char * property)
{
  if (*styleptr != style)
    {
      if (*styleptr)
        {
          if (GTK_WIDGET_REALIZED(GTK_WIDGET(scale)))
            gtk_style_detach (*styleptr);
          g_object_unref (*styleptr);
        }
      
      g_object_ref (G_OBJECT (style));
      
      if (GTK_WIDGET_REALIZED(GTK_WIDGET(scale)))
        *styleptr = gtk_style_attach (style, gtk_widget_get_parent_window (GTK_WIDGET (scale)));
      else
        *styleptr = style;
      
      g_object_notify (G_OBJECT (scale), property);
    }
}

void
gtk_audio_scale_set_zoom_upper_style		(GtkAudioScale * scale, GtkStyle * style)
{
  g_return_if_fail (GTK_IS_AUDIO_SCALE (scale));
  g_return_if_fail (GTK_IS_STYLE (style));
  
  gtk_audio_scale_internal_set_zoom_style (scale, style, &scale->zoom_upper_style, "zoom_upper_style");
}

GtkStyle *
gtk_audio_scale_get_zoom_lower_style 	(GtkAudioScale * scale)
{
  g_return_val_if_fail (GTK_IS_AUDIO_SCALE (scale), NULL);
  
  return scale->zoom_lower_style;
}

void
gtk_audio_scale_set_zoom_lower_style		(GtkAudioScale * scale, GtkStyle * style)
{
  g_return_if_fail (GTK_IS_AUDIO_SCALE (scale));
  g_return_if_fail (GTK_IS_STYLE (style));
  
  gtk_audio_scale_internal_set_zoom_style (scale, style, &scale->zoom_lower_style, "zoom_lower_style");
}

guint
gtk_audio_scale_get_zoom_indicator_size   	(GtkAudioScale * scale)
{
  g_return_val_if_fail (GTK_IS_AUDIO_SCALE (scale), 0);
  
  return scale->zoom_indicator_size;
}

void
gtk_audio_scale_set_zoom_indicator_size    	(GtkAudioScale * scale, guint size)
{
  g_return_if_fail (GTK_IS_AUDIO_SCALE (scale));
  
  scale->zoom_indicator_size = size;
  g_object_notify (G_OBJECT (scale), "zoom_indicator_size");
}

gdouble
gtk_audio_scale_get_value                      (GtkAudioScale * scale)
{
  g_return_val_if_fail (GTK_IS_AUDIO_SCALE (scale), 0.0);
  
  if (scale->logarithmic)
    return scale->adjustment->value;
  else
    return scale->adjustment->value;
}

void
gtk_audio_scale_set_value                         (GtkAudioScale * scale, gdouble value)
{
  g_return_if_fail (GTK_IS_AUDIO_SCALE (scale));
  
  if (scale->logarithmic)
    value = log (value);
  
  value = CLAMP (value, scale->adjustment->lower, scale->adjustment->upper);
  
  gtk_adjustment_set_value (scale->adjustment, value);
}

GtkOrientation
gtk_audio_scale_get_orientation         (GtkAudioScale * scale)
{
  g_return_val_if_fail (GTK_IS_AUDIO_SCALE (scale), GTK_ORIENTATION_HORIZONTAL);
  
  return scale->orientation;
}

void
gtk_audio_scale_set_orientation                   (GtkAudioScale * scale, GtkOrientation orient)
{
  g_return_if_fail (GTK_IS_AUDIO_SCALE (scale));
  
  scale->orientation = orient;
  g_object_notify (G_OBJECT (scale), "orientation");
}
